/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */


/** @file bdb_common.h
 * @brief The common interfaces of database
 * 
 * This file implements the common interfaces for creating, openning,
 * and closing database. The database here is built on the basis of
 * Berkeley-DB. The interfaces defined here will be called by other
 * database functions.
 */

#ifndef _BDB_COMMON_H_
#define _BDB_COMMON_H_

#include <db.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "xos.h"


/* for test program */
#ifdef _WIN32
extern int getopt(int, char *const *, const char *);
extern char *optarg;
#define snprintf _snprintf
#else
#include <unistd.h>
#endif
#define UDATA_FILE     "user_infos.txt"
#define GDATA_FILE     "group_infos.txt"

/** Default home directory of database */
#define DEFAULT_HOMEDIR	DB_DEFAULT_TMP_DIR
/* passwd database */
/** Default database name of PASSWD */
#define GPASSWD_DB	"GpasswdDB.db"
/** Default index database of PASSWD, dn as index value */
#define IGPASSWD_DB	"iGpasswdDB.db"
/** Default index database of PASSWD, uid as index value */
#define UGPASSWD_DB	"uGpasswdDB.db"

/* group database */
/** Default database name of GGROUPS */
#define GGROUP_DB	"GgroupDB.db"
/** Default index database of GGROUPS, subgroup as index value*/
#define IGGROUP_DB	"iGgroupDB.db"
/** Defautl index database of GGROUPS, gid as index value*/
#define UGGROUP_DB	"uGgroupDB.db"


/** flag of primary database */
#define PRIMARY_DB	0
/** flag of index database */
#define SECONDARY_DB	1

/** @brief structure of database set.
 * 
 * The structure holds all the handle of relative databases, 
 * which can be dealed with at the same time. 
 */
typedef struct stock_dbs {
	/** The pointer of primary database: GPASSWD_DB */
	DB *gpasswd_dbp;
	/** The pointer of index database:IGPASSWD_DB, using dn as index */
	DB *igpasswd_sdbp;
	/** The pointer of index database: UGPASSWD_DB, using uid as index */
	DB *ugpasswd_sdbp;

	/** The pointer of primary database: GGROUP_DB */
	DB *ggroup_dbp;
	/** The poinger of index database: IGGROUP_DB, using subgroup name as index */
	DB *iggroup_sdbp;
	/** The pointer of index database: UGGROUP_DB, using gid as index */
	DB *uggroup_sdbp;

	/** Home directory of databases */
	char *db_home_dir;
	/** Name of index dababase: IGPASSWD_DB*/
	char *ipasswd_db_name;
	/** Name of primary database: GPASSWD_DB */
	char *gpasswd_db_name;
	/** Name of index database: UGPASSWD_DB */
	char *upasswd_db_name;
	/** Name of index database: IGGROUP_DB */
	char *igroup_db_name;
	/** Name of primary database: GGROUP_DB */
	char *ggroup_db_name;
	/** Name of index database: UGGROUP_DB */
	char *ugroup_db_name;

} STOCK_DBS;



/** Close the set of databases.
 * 
 */
int databases_close(STOCK_DBS *);

/** Prepare all the databases, opening the primary database and connecting
 * index to primary
 */
int databases_setup(STOCK_DBS *, const char *, FILE *);

/** Initiatlize the values in STOCK_DBS struct 
 */
void initialize_stockdbs(STOCK_DBS *);

/** Open database
 */
int open_database(DB **, const char *, const char *, FILE *, int);

/** Identify all the files that will hold our databases 
 */
void set_db_filenames(STOCK_DBS *);

#endif				/* _BDB_COMMON_H_ */
