/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

#ifndef _BUFFER_H_
#define _BUFFER_H_

#include <sys/types.h>

#ifndef HAVE_U_CHAR
typedef unsigned char u_char;
# define HAVE_U_CHAR
#endif

#ifndef HAVE_U_INT
typedef unsigned int u_int;
#endif


typedef struct {
	u_char *buf;		/* Buffer for data. */
	u_int alloc;		/* Number of bytes allocated for data. */
	u_int offset;		/* Offset of first byte containing data. */
	u_int end;		/* Offset of last byte containing data. */
} Buffer;

void buffer_init(Buffer *);
void buffer_clear(Buffer *);
void buffer_free(Buffer *);

u_int buffer_len(Buffer *);
void *buffer_ptr(Buffer *);

void buffer_append(Buffer *, const void *, u_int);
void *buffer_append_space(Buffer *, u_int);

int buffer_check_alloc(Buffer *, u_int);

void buffer_get(Buffer *, void *, u_int);

void buffer_consume(Buffer *, u_int);
void buffer_consume_end(Buffer *, u_int);

void buffer_dump(Buffer *);

int buffer_get_ret(Buffer *, void *, u_int);
int buffer_consume_ret(Buffer *, u_int);
int buffer_consume_end_ret(Buffer *, u_int);

#include <openssl/bn.h>

void buffer_put_bignum(Buffer *, const BIGNUM *);
void buffer_put_bignum2(Buffer *, const BIGNUM *);
void buffer_get_bignum(Buffer *, BIGNUM *);
void buffer_get_bignum2(Buffer *, BIGNUM *);

u_short buffer_get_short(Buffer *);
void buffer_put_short(Buffer *, u_short);

u_int buffer_get_int(Buffer *);
void buffer_put_int(Buffer *, u_int);

u_int64_t buffer_get_int64(Buffer *);
void buffer_put_int64(Buffer *, u_int64_t);

int buffer_get_char(Buffer *);
void buffer_put_char(Buffer *, int);

void *buffer_get_string(Buffer *, u_int *);
void buffer_put_string(Buffer *, const void *, u_int);
void buffer_put_cstring(Buffer *, const char *);

#define buffer_skip_string(b) \
    do { u_int l = buffer_get_int(b); buffer_consume(b, l); } while (0)

int buffer_put_bignum_ret(Buffer *, const BIGNUM *);
int buffer_get_bignum_ret(Buffer *, BIGNUM *);
int buffer_put_bignum2_ret(Buffer *, const BIGNUM *);
int buffer_get_bignum2_ret(Buffer *, BIGNUM *);
int buffer_get_short_ret(u_short *, Buffer *);
int buffer_get_int_ret(u_int *, Buffer *);
int buffer_get_int64_ret(u_int64_t *, Buffer *);
void *buffer_get_string_ret(Buffer *, u_int *);
int buffer_get_char_ret(char *, Buffer *);

#endif				/* _BUFFER_H_ */
