/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */


/** @file xos_cert.h
 * @brief the interfaces are designed to manage certificate-related
 * operation.
 * 
 * */
#ifndef _XOS_CERT_H_
#define _XOS_CERT_H_

#include <openssl/x509.h>
#include <openssl/x509v3.h>

/** create a proxy by given user's certificate and user's private key.
 * @param usercert the user certificate.
 * @param userkey the key file holding user's private key
 * @return zero if sucessful, or non-zero
 * */
int xos_proxy_init_cmd(char *usercert, char *userkey, char *proxyfile);

/** delegate a new proxy by existing proxy certificate.
 * @param current_proxy  the file path of the existing proxy certificate
 * @pram next_proxy the file path to create next proxy 
 * @param days the valid days. The valid time is limited by issuer's certificate
 * 	and deleted proxy validity.
 * @return zero if successful, or non-zero.
 * */
int xos_proxy_delegate_cmd(char *current_proxy, char *next_proxy,
			   int days);

/** in-secue interface: Expired
 * Set attributes to proxy (in the field of nsCommend)
 * @param certifle the path of proxy file
 * @param keyfile the path of key file
 * @param days the valid time
 * @param attrs the attributes to be set
 * @pram proxyfile the path to create a new proxy holding given attributes.
 * @return zero if sucessful, or non-zero.
 * */
int xos_proxy_delegate_exec(char *certfile, char *keyfile, int days,
			    char *attrs, char *proxyfile);

/** in-secure interface: Expired 
 * fetch attributes from proxy.
 * @param proxy_file the path of proxy 
 * @pram attrs the buffer to store fetched attributes.
 * @return zero if sucessful, or non-zero.
 * */
int xos_cert_attribute_load(char *proxy_file, char *attrs);

/** Expired interface
 * */
int xos_cert_attribute_get(X509 * cert, char *attrs);

/** Expired interface
 * */
int xos_cert_expirytime_get(X509 * cert, time_t * not_after);

/** in-secure inteface:
 * create proxy with given attriburtes
 * @param certfile the buffer of certificate
 * @param keyfile the buffer of private key
 * @param days the valid time of proxy
 * @param attrs the attributes to be set in proxy
 * @param proxyfile the path to create proxy
 * @return zero if successful, or non-zero.
 * */
int create_proxy(char *certfile, char *keyfile, int days, char *attrs,
		 char *proxyfile);

/** get extension of certificate to buffer.
 * The interface will fetch all the extension from certificate.
 * @param cert the pointer of certificate
 * @param ext_buf the buffer to be stored in extension.
 * @return zero if successful, or non-zero.
 * */
int xos_cert_attribute_get_ax(X509 * cert, char *ext_buf);

/** get specific extension value by given attribute label.
 * The attribute label is defined with OID name according to CDA.
 * @param ext the buffer holding all the extension information from 
 * 	certifciate.
 * @param attr_label the OID used in CDA certificcate.
 * @param attr_value the buffer to return given value of OID.
 * @return zero if successful, or non-zero.
 * */
int xos_cert_extensions_getattr(char *ext, char *attr_label,
				char *attr_value);

#endif				/* _XOS_CERT_H_ */
