/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

/** @file xos_dbaux_tmp.h
 * @brief present a global database to store temporary information.
 *
 * The database is designed mainly for communication between PAM hooks
 * in the cases where they are used seperatedly. However, trusted
 * appliations can also call the client interfaces to stored the private
 * information. 
 *
 * The temporary data stored in tempoarry database can only be applied 
 * (retrieved) one time. The data will be removed after get operation.
 * 
 * */
#ifndef _XOS_DBAUX_TMP_H_
#define _XOS_DBAUX_TMP_H_

#include "xos.h"
#include "bdb_common.h"
#include "xos_dbaux.h"

/** Default directory to locate databases */
#define	TMPDB_DIR	DB_DEFAULT_TMP_DIR
/** the name of temporary databse */
#define	TMPINFO_DB	"tmpinfo.db"

/** the length of key to retrieve data */
#define KEY_LEN		32

/** data structure of database */
typedef struct _tmpdbs {
	/** pointer of database file */
	DB *tmpinfo_dbp;
	/** the home directory of database file */
	char *tmpdb_home_dir;
	/** the file name of database */
	char *tmpinfo_db_name;
} TMPINFO_DBS;

/** initiate database 
 * @param tmpdbp the pointer of data structure
 * @return zero if successful, or non-zero.
 * */
int tmpinfo_db_init(TMPINFO_DBS * tmpdbp);

/** close  the database
 * @param tmpdbp the pointer of data structure
 * @return zero if successful, or non-zero.
 * */
int tmpinfo_db_release(TMPINFO_DBS * tmpdbp);

/** get data from temporary database.
 * @param skey the buffer holding the key to retrieve data
 * @param databuf the pointer of returned data.
 * @param buf_len the length of return data.
 * @return zero if successful, or non-zero.
 * */
int tmpinfo_db_get(char *skey, char **databuf, int *buf_len);

/** put data to database
 * @param skey the buffer to returned key of stored data.
 * @param databuf the buffer holding the will-be stored data.
 * @return zero if successful, or non-zero. 
 * */
int tmpinfo_db_put(char *skey, char *databuf);

/** delete the data with given key
 * @param skey the buffer holding the key of deleted data
 * @return zero if successful, or non-zero.
 * */
int tmpinfo_db_delete(char *skey);

/** print all the records in database */
void tmpinfo_db_printdb();

/** clear all the records in database */
int tmpinfo_db_clearall();


#ifndef _XOS_DBAUX_H_

void create_sranddbkey(char *, int);
void print_error(int, const char *);
void init_DBT(DBT *, DBT *);
size_t pack_string(char *, char *, size_t);

#else

extern void create_sranddbkey(char *, int);
extern void print_error(int, const char *);
extern void init_DBT(DBT *, DBT *);
extern size_t pack_string(char *, char *, size_t);

#endif

#endif				/* _XOS_DBAUX_TMP_H_ */
