/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

#ifndef _KEY_H
#define _KEY_H

#include <openssl/rsa.h>
#include <openssl/dsa.h>
#include <openssl/evp.h>

typedef struct Key Key;
enum types {
	KEY_RSA1,
	KEY_RSA,
	KEY_DSA,
	KEY_UNSPEC
};
enum fp_type {
	SSH_FP_SHA1,
	SSH_FP_MD5
};
enum fp_rep {
	SSH_FP_HEX,
	SSH_FP_BUBBLEBABBLE
};

/* key is stored in external hardware */
#define KEY_FLAG_EXT		0x0001

struct Key {
	int type;
	int flags;
	RSA *rsa;
	DSA *dsa;
};

typedef struct {

	unsigned char key[EVP_MAX_KEY_LENGTH];
	unsigned char iv[EVP_MAX_KEY_LENGTH];

} SymKey;


Key *key_new(int);
Key *key_new_private(int);
void key_free(Key *);
Key *key_demote(const Key *);
int key_equal(const Key *, const Key *);
char *key_fingerprint(const Key *, enum fp_type, enum fp_rep);
u_char *key_fingerprint_raw(const Key *, enum fp_type, u_int *);
const char *key_type(const Key *);
int key_write(const Key *, FILE *);
int key_read(Key *, char **);
u_int key_size(const Key *);

Key *key_generate(int, u_int);
Key *key_from_private(const Key *);
int key_type_from_name(char *);

Key *key_from_blob(const u_char *, u_int);
int key_to_blob(const Key *, u_char **, u_int *);
const char *xos_keyname(const Key *);
int key_names_valid2(const char *);

int xos_key_sign(const Key *, u_char **, u_int *, const u_char *, u_int);
int xos_key_verify(const Key *, const u_char *, u_int, const u_char *,
		   u_int);

int xos_dss_sign(const Key *, u_char **, u_int *, const u_char *, u_int);
int xos_dss_verify(const Key *, const u_char *, u_int, const u_char *,
		   u_int);
int xos_rsa_sign(const Key *, u_char **, u_int *, const u_char *, u_int);
int xos_rsa_verify(const Key *, const u_char *, u_int, const u_char *,
		   u_int);

void xos_symkey_generate(SymKey * k);
void xos_symkey_print(SymKey * k);
#endif				/* _KEY_H */
