/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

/** @file xos_policy_mgmt.h
 * @brief the interfaces for policy manamgement
 * */
#ifndef _XOS_POLICY_MGMT_H_
#define _XOS_POLICY_MGMT_H_

#include "xos.h"
#include "xos_policy_rule.h"
#include "xos_dbaux.h"
#include "xos_reslimit.h"


#define XPM_MAXLEN	MAXFIELD


/** reconstruct a standard regular expression
 * @param regex_old the exression to be reconstructed.
 * @param regex_new the new regular expression.
 * @return  zero if successful, or non-zero.
 * */
int regex_replace(char *regex_old, char **regex_new);

/** matching produce
 * @param rtype the type of rule
 * @param matched_sub the subject to be matched
 * @param matched_rule the rule matching given subject.
 * @return zero if successful, or non-zero.
 * */
int xos_policy_match(enum XPLY_RT rtype,
		     struct subject_t *matched_sub,
		     struct rule_t *matched_rule);


/** find the matching rule
 * @param rtype the type of rule
 * @param matched_sub the subject to be matched
 * @param ret_rule the rule matching given subject.
 * @param ert_handler the handler of matching rule
 * @return zero if successful, or non-zero.
 * */

int xos_policy_findrule(enum XPLY_RT rtype,
			struct subject_t *sub, struct rule_t **ret_rule,
			char **ret_handler);

#if 0

/* use xos_policy_mgmt_1.c and xos_policy_mgmt_2.c */
int xos_policy_pwdbinding(GPASSWD * gpwd);
int xos_policy_grpbinding(GGROUPS * ggrp);

#else

/* use xos_policy_mgmt_3.c */

/** apply rules to given user account
 * @param gpwd the user's GPASSWD data.
 * @param ru_rules the buffer to hold the applied rules
 * @return zero if successful, or non-zero.
 * */
int xos_policy_pwdbinding(GPASSWD * gpwd, char *ru_rules);

/** apply rules to given user group
 * @param ggrp the group structure.
 * @param ru_rules the buffer to hold the appled rules.
 * @return zero if successful, or non-zero.
 * */
int xos_policy_grpbinding(GGROUPS * ggrp, char *ru_rules);


#endif

/** relaase the user from all policy rules
 * @param gpwd the user identity
 * @return zero if successful, or non-zero
 * */
int xos_policy_pwdunbinding(GPASSWD * gpwd);

/** release the group from all policy rules
 * @param ggrp the group 
 * @return zero if successful, or non-zero
 * */
int xos_policy_grpunbinding(GGROUPS * ggrp);

/** apply the rule to all current VO users
 * @param updated_rule the rule to by applied
 * @return zero if successful, or non-zero
 * */
int xos_policy_apply(struct rule_t *updated_rule);


/** setting mapping boundaries of uid and gid
 * @param uid_max the uppest boundary of uid
 * @param uid_min the lowest boundary of uid
 * @param gid_max the uppest boundary of gid
 * @param gid_min the lowest boundary of gid
 * @return zero if successful, or non-zero
 * */
int xos_policy_vosetting(int uid_max, int uid_min, int gid_max,
			 int gid_min);

/** add a new policy rule
 * @param new_rule the new rule needed to be added
 * @param rhandler the returned handler of new rule
 * @return zero if successful, or non-zero
 * 
 * */
int xos_policy_addrule(struct rule_t *new_rule, char **rhandler);

/** remove a given rule
 * @param rhandler the handler of removed rule
 * @return zero if successful, or non-zero
 * */
int xos_policy_removerule(char *rhandler);

/** modify given rule
 * @param rhandler the handler of rule
 * @param new_rule the new rule
 * @return zero if successful, or non-zero.
 * */
int xos_policy_modifyrule(char *rhandler, struct rule_t *new_rule);

/** search the given rule in database
 * @param rtype the type of searched rule
 * @param r_sub the subject of searched rule
 * @param r_obj the object of searched rule
 * @param ret_rule if the matching rule is fuond, return the pointer of rule
 * @param ret_handler the handler of returned rule.
 * @return zero if successful, or non-zero.
 * */
int xos_policy_searchrule(enum XPLY_RT rtype, struct subject_t *r_sub,
			  struct object_t *r_obj, struct rule_t **ret_rule,
			  char **ret_handler);

/** check whether given subject is in blacklist
 * @param r_sub the subject of user
 * @param in_blacklist if the user is in blacklist, the param will return 1,
 * 	or zero.
 * @return zero if successful, or non-zero.
 * */
int xos_policy_ACreject(struct subject_t *r_sub, int *in_blacklist);

/** adjust the priority of rule
 * @param from the priority (or location) of current rule
 * @param to the priority wanted to be adjusted.
 * @return zero if successful, or non-zoer.
 * */
int xos_policy_setriority(int from, int to);

#endif				/* _XOS_POLICY_MGMT_H_ */
