/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

#ifndef XOS_POLICY_H
#define XOS_POLICY_H

#include <time.h>
#include "xos.h"

#define XPLY_MAXLEN	DATAFIELD_MAXLEN

/** type of policy rule */
enum XPLY_RT {
	XPLY_RT_NUL,	/**< NULL - invalid, for initiate */
	XPLY_RT_MA,	/**< MA - Mapping */
	XPLY_RT_AC,	/**< AC - Access Control */
	XPLY_RT_RU,	/**< RU - Resource usage */
	XPLY_RT_AP	/**< AP - Application Specific */
};

/** type of subject */
enum XPLY_ST {
	XPLY_ST_NUL,	/**< NULL - invalid subject, for initiate */
	XPLY_ST_ACT,	/**< ACT - DN,VO,ROLE of VO user, for account mapping */
	XPLY_ST_GRP	/**< GRP - attributes of VO user, for group mapping */
};

/** type of object */
enum XPLY_OT {
	XPLY_OT_NUL,	/**< NULL - invalid object, for initiate */
	XPLY_OT_ACT,	/**< ACT - local username or '*', for account mapping */
	XPLY_OT_GRP,	/**< GRP - local groupname of '*', for group mapping */
	XPLY_OT_FILE,	/**< FILE - object is used in AC rule*/
	XPLY_OT_NODE,	/**< NODE - object is used in AC rule*/
	XPLY_OT_QUOTA	/**< QUOTA - object is used in quota rule*/
};

/** type of quota */
enum XPLY_QT {
	XPLY_QT_ABS,	/**< ABS - absolute value */
	XPLY_QT_REL	/**< REL - relative value */
};

/** type of resource for enforcement */
enum XPLY_REST {
	XPLY_REST_CPU,		/**< CPU - CPU resource */
	XPLY_REST_MEM,		/**< MEM - Memory resource */
	XPLY_REST_DISK,		/**< DISK - Disk resource */
	XPLY_REST_NETWORK,	/**< NETWORK - network bandwidth */
	XPLY_REST_NPROC,	/**< NPROC - maximun number of open process */
	XPLY_REST_NOFILE	/**< NOFILE - maximum number of open files */
};

///** type of rule status */
//enum XPLY_RS {
//      XPLY_RS_APP,    /**< APPEND - rule is not applied currently */
//      XPLY_RS_EXE,    /**< EXECUTE - rule is used now */
//      XPLY_RS_MOD     /**< MODIFIED - rule has been modified but not been applied*/
//};

#define XPLY_PERM_T		unsigned int
#define XPLY_PERM_READ		0x0001
#define	XPLY_PERM_WRITE		0x0010
#define	XPLY_PERM_EXE		0x0100


/** data definition of subject */
struct subject_t {
	enum XPLY_ST sub_type;
	char sub_content_1[XPLY_MAXLEN];
	char sub_content_2[XPLY_MAXLEN];
};


/** data definition of object */
struct object_t {
	enum XPLY_OT obj_type;
	union {
		char objc_object[XPLY_MAXLEN];
		enum XPLY_REST objc_restype;
	} obj_content;
};


/** data definition of quota */
struct quota_t {
	enum XPLY_QT quo_type;
	float quo_val;
};


/** data definition of driver */
struct driver_t {
	char driver_name[XPLY_MAXLEN];
	char driver_params[XPLY_MAXLEN];
};


/** data definition of policy rule */
struct rule_t {
	enum XPLY_RT r_type;

	struct subject_t r_subject;	/**< subject applied by the rule */
	struct object_t r_object;	/**< object applied by the rule */
	union {
		XPLY_PERM_T rc_perm;		/**< for AC rule */
		struct quota_t rc_quota;	/**< for RU rule */
//              struct driver_t rc_app_driver;  /**< for AP rule */
//              struct driver_t rc_ma_driver;   /**< for MA rule */
		struct driver_t rc_driver;	/**< for MA/AP rule */
	} r_content;	/**< the content of rule */

	/** indicate if the rule is disabled (forbidden?) */
	int r_valid;

	/** indicate the period when the rule is applied */
	time_t r_time;

	/** indicate the current status of rule*/
	//enum XPLY_RS r_status;
};

#endif				/*XOS_POLICY_H */
