/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

/** @file xos_reslimit.h
 * @brief Policy management in PAM module
 *
 * The file defines all the handlers abount policy management in
 * PAM module. Currently, the codes are implemmented to limit
 * resouce quota. The strategy is to use setrlimit() to finish
 * jobs.
 *
 * \todo Other policy management will be developed and combie together 
 * with resource quota limitation.
 */

#ifndef _XOS_RESLIMIT_H_
#define _XOS_RESLIMIT_H_

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/resource.h>
#include <limits.h>
#include "xos.h"

#define LIMIT_LOGIN 		RLIM_NLIMITS+1
#define LIMIT_NUMSYSLOGINS 	RLIM_NLIMITS+2
#define LIMIT_PRI 		RLIM_NLIMITS+3

#define LIMIT_SOFT  		1
#define LIMIT_HARD  		2


/* Module defines */
#define LINE_LENGTH 		1024

#define LIMITS_DEF_USER     	0	/**< limit set by an user entry */
#define LIMITS_DEF_GROUP    	1	/**< limit set by a group entry */
#define LIMITS_DEF_ALLGROUP 	2	/**< limit set by a group entry */
#define LIMITS_DEF_ALL      	3	/**< limit set by an default entry */
#define LIMITS_DEF_DEFAULT  	4	/**< limit set by an default entry */
#define LIMITS_DEF_NONE     	5	/**< limit not set yet */


#define LIMITED_OK 		0	/**< limit for setting appeared to work */
#define LIMIT_ERR  		1	/**< error for setting a limit */
#define LOGIN_ERR  		2	/**< error for too many logins */

#define VO_QUOTiA_CONF		LOCAL_QUOTA_CONF



/** @brief structure of user's limitation
 */
struct user_limits_struct {
	int supported;
	int src_soft;
	int src_hard;
	struct rlimit limit;
};

/* internal data */
/** @brief structure holding the limit paramters
 */
struct _limit_s {
	int login_limit;	/* the max logins limit */
	int login_limit_def;	/* which entry set the login limit */
	int flag_numsyslogins;	/* whether to limit logins only for a
				   specific user or to count all logins */
	int priority;		/* the priority to run user process with */
	struct user_limits_struct limits[RLIM_NLIMITS];
	char conf_file[BUFSIZ];
	int utmp_after_pam_call;
	char login_group[LINE_LENGTH];
};

typedef struct _limit_s XLIMIT;

int xos_reslimit_ismember(char *user, char *group);
int xos_reslimit_init(XLIMIT * pl);
void xos_reslimit_quota(int source,
			const char *lim_type, const char *lim_item,
			const char *lim_value, XLIMIT * pl);

int xos_reslimit_setup(const char *uname, uid_t uid, XLIMIT * pl);



#endif				/* _XOS_PLYMGT_H_ */
