/*
 * Copyright (c) 2008 Institute of Computing Technology of Chinese Academy of Sciences and Telefónica Investigación y Desarrollo SAU.
 * All rights reserved.
 *
 * This file is part of Linux-XOS-VOsupport, which is part of XtreemOS,
 * a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details.
 * The XtreemOS project has been developed with the financial support of the European
 * Commission's IST program under contract #FP6-033576.
 *
 * Linux-XOS-VOsupport is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 2 of the License, or (at your option) any later version.
 *
 * Linux-XOS-VOsupport is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with LinuxMD-XOS.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * */

/** @file xos_rquota.h
 * @brief manage the local resource quota.
 *
 * The intefaces presented here are only used to manage
 * local quota setting stored in given file: quota.conf.
 * The file provides an opportutity to local system 
 * administrator to customize local resource setting.
 * 
 * */

#ifndef _XOS_RQUOTA_H_
#define _XOS_RQUOTA_H_

#include "xos.h"


#define	QUOTA_DIR	DB_DEFAULT_TMP_DIR
#define QUOTA_FILE 	"quota.conf"

#define POLICY_DIR	QUOTA_DIR
#define POLICY_FILE	"policy.conf"


#define QUOTA_CHCPU	1
#define QUOTA_CHMEM	2
#define QUOTA_CHDISK	4

typedef struct {
	char *rname;  	/**< subject name*/
	int f_group; 	/**< 1: group, 0: account*/
	long rcpu;	/**< CPU quota*/
	int f_cpu;
	long rmem;	/**< MEM quota */
	int f_mem;
	long rdisk;	/**< DISK quota*/
	int f_disk;
} ResQuota;

/** initiate the quota structure
 * @param rquota the pointer of quota
 * @return zero if successful, or non-zero
 * */
int xos_resquota_init(ResQuota * rquota);

/** setup the quota before PAM session open
 * @param rquota the pointer of quota
 * @return zero if successful, or non-zero
 *  
 * */
int xos_resquota_setup(ResQuota * rquota);

/** clear the quota
 * @param rquota the pointer of quota
 * @return zero if successful, or non-zero
 * */
int xos_resquota_clear(ResQuota * rquota);

/** modify the quota
 * @param rquota the pointer of quota
 * @return zero if successful, or non-zero
 *  
 * */
int xos_resquota_modify(ResQuota * newrquota, int flags);

/** expried function
 * */
int xos_resquota_interpret(char *role, ResQuota * rquota);

#endif				/* _XOS_RQUOTA_H_ */
